@@ -1,17 +1,20 @@
 import asyncio
+import copy
 import logging
 from datetime import datetime, timezone
 
+from fastapi_websocket_rpc import RpcChannel
 import openpectus.aggregator.models as Mdl
 import openpectus.protocol.aggregator_messages as AM
 import openpectus.protocol.engine_messages as EM
 import openpectus.protocol.messages as M
 from openpectus.aggregator.data import database
 from openpectus.aggregator.data.repository import RecentRunRepository, PlotLogRepository, RecentEngineRepository
-from openpectus.aggregator.frontend_publisher import FrontendPublisher
+from openpectus.aggregator.frontend_publisher import FrontendPublisher, PubSubTopic
 from openpectus.aggregator.models import EngineData
 from openpectus.protocol.aggregator_dispatcher import AggregatorDispatcher
 from openpectus.protocol.models import SystemTagName, MethodStatusEnum
+from openpectus.aggregator.exceptions import AggregatorCallerException, AggregatorInternalException
 
 logger = logging.getLogger(__name__)
 
@@ -133,7 +136,7 @@ class FromEngine:
                 engine_data.reset_run()
             else:
                 engine_data.run_data.runlog = msg.runlog
-                engine_data.method = msg.method
+                engine_data.method_state = msg.method_state
                 try:
                     recent_run_repo.store_recent_run(engine_data)
                     logger.info(f"Stored recent run {_run_id=}")
@@ -309,29 +312,47 @@ class FromEngine:
 
 
 class FromFrontend:
-    def __init__(self, engine_data_map: EngineDataMap, dispatcher: AggregatorDispatcher):
+    def __init__(self, engine_data_map: EngineDataMap, dispatcher: AggregatorDispatcher, publisher: FrontendPublisher):
         self._engine_data_map = engine_data_map
         self.dispatcher = dispatcher
+        self.publisher = publisher
+        self.dead_man_switch_user_ids: dict[str, str] = dict()
+        self.publisher.register_on_disconnect(self.on_ws_disconnect)
+        self.publisher.pubsub_endpoint.methods.event_notifier.register_subscribe_event(self.user_subscribed_pubsub)  # type: ignore
 
     def __str__(self) -> str:
         return f'{self.__class__.__name__}(engine_data_map={self._engine_data_map}, dispatcher={self.dispatcher})'
 
-    async def method_saved(self, engine_id: str, method: Mdl.Method, user_name: str) -> bool:
+    async def method_saved(self, engine_id: str, method: Mdl.Method, user_name: str) -> int:
+        existing_version = self._engine_data_map[engine_id].method.version
+        version_to_overwrite = method.version
+        logger.debug(f"Save method version: {method.version}")
+        if existing_version != version_to_overwrite:
+            raise AggregatorCallerException(f"Method version mismatch: trying to overwrite version {version_to_overwrite}" +
+                                            f"when existing version is {existing_version}")
+        # Take shallow copy and increment version
+        new_method = copy.copy(method)
+        new_method.version += 1
+
         try:
-            response = await self.dispatcher.rpc_call(engine_id, message=AM.MethodMsg(method=method))
+            response = await self.dispatcher.rpc_call(engine_id, message=AM.MethodMsg(method=new_method))
             if isinstance(response, M.ErrorMessage):
                 logger.error(f"Failed to set method. Engine response: {response.message}")
-                return False
-        except Exception:
+                if response.caller_error:
+                    raise AggregatorCallerException(response.message)
+                else:
+                    raise AggregatorInternalException(response.message)
+        except Exception as e:
             logger.error("Failed to set method", exc_info=True)
-            return False
+            raise e
 
         # update local method state
         engine_data = self._engine_data_map.get(engine_id)
         if engine_data is not None:
-            engine_data.method = method
+            engine_data.method = new_method
             engine_data.contributors.add(user_name)
-        return True
+            asyncio.create_task(self.publisher.publish_method_changed(engine_id))
+        return new_method.version
 
     async def request_cancel(self, engine_id, line_id: str, user_name: str) -> bool:
         engine_data = self._engine_data_map.get(engine_id)
@@ -365,13 +386,51 @@ class FromFrontend:
         engine_data.contributors.add(user_name)
         return True
 
+    def get_dead_man_switch_user_ids(self, topics: list[str]):
+        return (topic.split("/")[1] for topic in topics if topic.startswith(PubSubTopic.DEAD_MAN_SWITCH))
+
+    async def user_subscribed_pubsub(self, subscriber_id: str, topics: list[str]):
+        for user_id in self.get_dead_man_switch_user_ids(topics):
+            self.dead_man_switch_user_ids[subscriber_id] = user_id
+
+    async def on_ws_disconnect(self, subscriber_id: str):
+        user_id = self.dead_man_switch_user_ids[subscriber_id]
+        user_has_other_dead_man_switch = len([other_user_id for other_user_id in self.dead_man_switch_user_ids.values() if other_user_id == user_id]) > 1
+        if(user_has_other_dead_man_switch): return
+        for engine_data in self._engine_data_map.values():
+            popped_user_id = engine_data.active_users.pop(user_id, None)
+            if(popped_user_id != None):
+                asyncio.create_task(self.publisher.publish_active_users_changed(engine_data.engine_id))
+
+    async def register_active_user(self, engine_id: str, user_id: str, user_name: str):
+        engine_data = self._engine_data_map.get(engine_id)
+        if engine_data is None:
+            logger.warning(f"Cannot register active user, engine {engine_id} not found")
+            return False
+        engine_data.active_users[user_id] = (Mdl.ActiveUser(
+            id=user_id,
+            name=user_name,
+        ))
+        asyncio.create_task(self.publisher.publish_active_users_changed(engine_id))
+        return True
+
+    async def unregister_active_user(self, engine_id: str, user_id: str):
+        engine_data = self._engine_data_map.get(engine_id)
+        if engine_data is None:
+            logger.warning(f"Cannot unregister active user, engine {engine_id} not found")
+            return False
+        if engine_data.active_users.pop(user_id, None) is None:
+            logger.warning(f"Couldn't unregister active user, user with id {user_id} was not found")
+            return False
+        asyncio.create_task(self.publisher.publish_active_users_changed(engine_id))
+        return True
 
 class Aggregator:
     def __init__(self, dispatcher: AggregatorDispatcher, publisher: FrontendPublisher, secret: str = "") -> None:
         self._engine_data_map: EngineDataMap = {}
         """ all client data except channels, indexed by engine_id """
         self.dispatcher = dispatcher
-        self.from_frontend = FromFrontend(self._engine_data_map, dispatcher)
+        self.from_frontend = FromFrontend(self._engine_data_map, dispatcher, publisher)
         self.from_engine = FromEngine(self._engine_data_map, publisher)
         self.secret = secret
 
