@@ -4,10 +4,10 @@ import threading
 import time
 import unittest
 from typing import Any, Generator
-from openpectus.lang.exec.errors import UodValidationError
+from openpectus.lang.exec.errors import EngineError, UodValidationError
+from openpectus.lang.exec.regex import RegexNumber
 from openpectus.lang.exec.tags_impl import ReadingTag, SelectTag
 
-import openpectus.protocol.models as Mdl
 import pint
 from openpectus.engine.engine import Engine
 from openpectus.engine.hardware import HardwareLayerBase, Register, RegisterDirection
@@ -18,8 +18,7 @@ from openpectus.lang.exec.uod import (
     UnitOperationDefinitionBase,
     UodCommand,
     UodBuilder,
-    RegexNamedArgumentParser,
-    RegexNumber,
+    RegexNamedArgumentParser
 )
 from openpectus.test.engine.utility_methods import (
     continue_engine, run_engine,
@@ -31,8 +30,6 @@ from typing_extensions import override
 
 configure_test_logger()
 set_engine_debug_logging()
-
-
 logging.getLogger("openpectus.lang.exec.pinterpreter").setLevel(logging.INFO)
 
 # pint takes forever to initialize - long enough
@@ -48,7 +45,7 @@ def get_queue_items(q) -> list[Tag]:
     return items
 
 
-def create_test_uod() -> UnitOperationDefinitionBase:
+def create_test_uod() -> UnitOperationDefinitionBase:  # noqa C901
     def reset(cmd: UodCommand, **kvargs) -> None:
         count = cmd.get_iteration_count()
         max_ticks = 5
@@ -293,7 +290,7 @@ class TestEngine(unittest.TestCase):
     def test_uod_command_w_arguments_fail(self):
         e = self.engine
         program = "CmdWithArgs: FAIL"
-        with self.assertRaises(ValueError):
+        with self.assertRaises(EngineError):
             run_engine(e, program, 10)
         self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.ERROR)
 
@@ -421,7 +418,7 @@ class TestEngine(unittest.TestCase):
 
         test('no_args', False)
 
-    def test_uod_invoke_command_signatures(self):
+    def test_uod_invoke_command_signatures(self):  # noqa C901
 
         def no_args(cmd: UodCommand):
             pass
@@ -574,7 +571,6 @@ Mark: X
 
         print_runtime_records(e)
 
-        self.assertEqual(3, len(rs))
         self.assertEqual("Block: A", rs[2].name)
         self.assertFalse(rs[0].has_state(RuntimeRecordStateEnum.Started))
 
@@ -666,21 +662,13 @@ Mark: C
         self.assertEqual(['A', 'B', 'C'], e.interpreter.get_marks())
 
     def test_set_and_continue_program(self):
-        program = """
-Mark: A
-"""
+        code = "Mark: A\nMark: B\nMark: C"
         e = self.engine
-        run_engine(e, program, 10)
-        self.assertEqual(['A', ], e.interpreter.get_marks())
-
-        program = """
-Mark: B
-Mark: C
-"""
-        e.set_method(Mdl.Method.from_pcode(pcode=program))
+        run_engine(e, code, 3)
+        self.assertEqual(['A'], e.interpreter.get_marks())
         continue_engine(e, 10)
 
-        self.assertEqual(['B', 'C'], e.interpreter.get_marks())
+        self.assertEqual(['A', 'B', 'C'], e.interpreter.get_marks())
 
     @unittest.skip("not implemented, needs input")
     def test_get_status(self):
@@ -950,7 +938,7 @@ Mark: A
 Restart
 """
         e = self.engine
-        run_engine(e, program, 4)
+        run_engine(e, program, 3)
 
         # when no commands need to be stopped, restart immediately moves to Stopped
         system_state = e.tags[SystemTagName.SYSTEM_STATE]
@@ -960,17 +948,17 @@ Restart
     # --- Totalizers ---
 
 
-    def test_totalizer_base_units_no_accumulator(self):
+    def test_totalizer_base_units_no_accumulator_allows_time_unit(self):
         e = self.engine
 
-        with self.subTest("allows_time_unit"):
-            run_engine(e, "Base: s\n0.1 Mark: A", 5)
-            self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.OK)
+        run_engine(e, "Base: s\n0.1 Mark: A", 5)
+        self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.OK)
 
-        with self.subTest("disallows_volume_unit"):
-            with self.assertRaises(ValueError):
-                run_engine(e, "Base: L\n0.1 Mark: A", 5)
-            self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.ERROR)
+    def test_totalizer_base_units_no_accumulator_disallows_volume_unit(self):
+        e = self.engine
+        with self.assertRaises(EngineError):
+            run_engine(e, "Base: L\n0.1 Mark: A", 5)
+        self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.ERROR)
 
     def test_totalizer_base_units_with_accumulator_volume(self):
         self.engine.cleanup()  # dispose the test default engine
@@ -997,7 +985,7 @@ Restart
 
         with self.subTest("disallows_cv_unit"):
             with create_engine_context(uod) as e:
-                with self.assertRaises(ValueError):
+                with self.assertRaises(EngineError):
                     run_engine(e, "Base: CV\n0.1 Mark: A", 5)
                 self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.ERROR)
 
@@ -1024,7 +1012,7 @@ Restart
 
         with self.subTest("disallows_volume_unit"):
             with create_engine_context(uod) as e:
-                with self.assertRaises(ValueError):
+                with self.assertRaises(EngineError):
                     run_engine(e, "Base: L\n0.1 Mark: A", 5)
                 self.assertEqual(e.tags[SystemTagName.METHOD_STATUS].get_value(), MethodStatusEnum.ERROR)
 
@@ -1067,7 +1055,7 @@ Base: s
 
     def test_accumulated_block_volume(self):
         self.engine.cleanup()  # dispose the test default engine
-
+        delta = 0.15
         set_interpreter_debug_logging()
 
         uod = (UodBuilder()
@@ -1082,8 +1070,9 @@ Base: s
 
         program = """
 Base: s
+Wait: 0.45s
 Block: A
-    0.45 End block
+    0.85 End block
 0.45 Mark: A
         """
         with create_engine_context(uod) as e:
@@ -1099,27 +1088,33 @@ Block: A
             self.assertEqual(block_vol.as_float(), 0.0)
             self.assertEqual(block_vol.unit, "L")
 
-            continue_engine(e, 2)  # Blank + Base
+            continue_engine(e, 1)  # Base
             self.assertEqual(block.get_value(), None)
-            self.assertAlmostEqual(acc_vol.as_float(), 0.2, delta=0.1)
-            self.assertAlmostEqual(block_vol.as_float(), 0.2, delta=0.1)
+            self.assertAlmostEqual(acc_vol.as_float(), 0.1, delta=delta)
+            self.assertAlmostEqual(block_vol.as_float(), 0.1, delta=delta)
+
+            continue_engine(e, 5)  # Wait
+            self.assertEqual(block.get_value(), None)
+            self.assertAlmostEqual(acc_vol.as_float(), 0.7, delta=delta)
+            self.assertAlmostEqual(block_vol.as_float(), 0.7, delta=delta)
+
 
             continue_engine(e, 1)  # Block
             self.assertEqual(block.get_value(), "A")
-            self.assertAlmostEqual(acc_vol.as_float(), 0.3, delta=0.1)
-            self.assertAlmostEqual(block_vol.as_float(), 0.1, delta=0.1)
+            self.assertAlmostEqual(acc_vol.as_float(), 0.8, delta=delta)
+            self.assertAlmostEqual(block_vol.as_float(), 0.1, delta=delta)
 
-            continue_engine(e, 5)
+            continue_engine(e, 8)
             self.assertEqual(block.get_value(), "A")
-            self.assertAlmostEqual(acc_vol.as_float(), 0.8, delta=0.1)
-            self.assertAlmostEqual(block_vol.as_float(), 0.6, delta=0.1)
+            self.assertAlmostEqual(acc_vol.as_float(), 1.6, delta=delta)
+            self.assertAlmostEqual(block_vol.as_float(), 0.9, delta=delta)
 
             continue_engine(e, 1)
             self.assertEqual(block.get_value(), None)
             # acc_vol keeps counting
-            self.assertAlmostEqual(acc_vol.as_float(), 0.9, delta=0.1)
+            self.assertAlmostEqual(acc_vol.as_float(), 1.7, delta=delta)
             # block_vol is reset to value before block A - so it matches acc_vol again
-            self.assertAlmostEqual(block_vol.as_float(), 0.9, delta=0.1)
+            self.assertAlmostEqual(block_vol.as_float(), 1.7, delta=delta)
 
     def test_accumulated_column_volume(self):
         self.engine.cleanup()  # dispose the test default engine
@@ -1194,47 +1189,8 @@ Base: CV
             self.assertAlmostEqual(acc_cv.as_float(), 0.6, delta=0.1)
             self.assertEqual(['A'], e.interpreter.get_marks())
 
-    def test_accumulated_column_volume_watch(self):
-        self.engine.cleanup()  # dispose the test default engine
-
-        uod = (UodBuilder()
-               .with_instrument("TestUod")
-               .with_author("Test Author", "test@openpectus.org")
-               .with_filename(__file__)
-               .with_hardware(TestHW())
-               .with_location("Test location")
-               .with_tag(ReadingTag("CV", "L"))
-               .with_tag(CalculatedLinearTag("calc", "L"))
-               .with_accumulated_cv(cv_tag_name="CV", totalizer_tag_name="calc")
-               .build())
-
-        program = """
-Base: CV
-Watch: Accumulated CV > 0.5 CV
-    Mark: A
-"""
-        with create_engine_context(uod) as e:
-            cv = e.tags["CV"]
-            cv.set_value(2.0, 0)
-            acc_cv = e.tags[SystemTagName.ACCUMULATED_CV]
-            run_engine(e, program, 1)
-
-            self.assertEqual(acc_cv.as_float(), 0.0)
-            self.assertEqual(acc_cv.unit, "CV")
-
-            continue_engine(e, 4)
-            self.assertAlmostEqual(acc_cv.as_float(), 0.2, delta=0.1)
-            self.assertEqual([], e.interpreter.get_marks())
-
-            continue_engine(e, 4)
-            self.assertAlmostEqual(acc_cv.as_float(), 0.4, delta=0.1)
-            self.assertEqual([], e.interpreter.get_marks())
-
-            continue_engine(e, 4)
-            self.assertAlmostEqual(acc_cv.as_float(), 0.6, delta=0.1)
-            self.assertEqual(['A'], e.interpreter.get_marks())
-
     def test_accumulated_column_block_volume(self):
+        # TODO flaky - even in dev
         self.engine.cleanup()  # dispose the test default engine
 
         uod = (UodBuilder()
@@ -1267,27 +1223,27 @@ Block: A
             self.assertEqual(block_cv.as_float(), 0.0)
             self.assertEqual(block_cv.unit, "CV")
 
-            continue_engine(e, 2)  # Blank + Base
+            continue_engine(e, 1)  # Base
             self.assertEqual(block.get_value(), None)
             self.assertAlmostEqual(acc_cv.as_float(), 0.2/2, delta=0.1)
             self.assertAlmostEqual(block_cv.as_float(), 0.2/2, delta=0.1)
 
-            continue_engine(e, 1)  # Block
+            continue_engine(e, 2)  # Block
             self.assertEqual(block.get_value(), "A")
             self.assertAlmostEqual(acc_cv.as_float(), 0.3/2, delta=0.1)
             self.assertAlmostEqual(block_cv.as_float(), 0.1/2, delta=0.1)
 
-            continue_engine(e, 5)
+            continue_engine(e, 3)
             self.assertEqual(block.get_value(), "A")
-            self.assertAlmostEqual(acc_cv.as_float(), 0.8/2, delta=0.1)
-            self.assertAlmostEqual(block_cv.as_float(), 0.6/2, delta=0.1)
+            self.assertAlmostEqual(acc_cv.as_float(), 0.7/2, delta=0.1)
+            self.assertAlmostEqual(block_cv.as_float(), 0.5/2, delta=0.1)
 
             continue_engine(e, 1)
             self.assertEqual(block.get_value(), None)
             # acc_vol keeps counting
-            self.assertAlmostEqual(acc_cv.as_float(), 0.9/2, delta=0.1)
+            self.assertAlmostEqual(acc_cv.as_float(), 0.8/2, delta=0.1)
             # block_vol is reset to value before block A - so it matches acc_vol again
-            self.assertAlmostEqual(block_cv.as_float(), 0.9/2, delta=0.1)
+            self.assertAlmostEqual(block_cv.as_float(), 0.8/2, delta=0.1)
 
 
     # --- Units ---
@@ -1330,7 +1286,7 @@ Restart
         e = self.engine
         self.assertEqual(0, e.tags[SystemTagName.RUN_COUNTER].as_number())
 
-        run_engine(e, program, 5)
+        run_engine(e, program, 4)
 
         run_id_1 = e.tags[SystemTagName.RUN_ID].get_value()
         self.assertEqual(e.tags[SystemTagName.SYSTEM_STATE].get_value(), SystemStateEnum.Restarting)
@@ -1379,9 +1335,7 @@ Reset
 Restart
 """
         e = self.engine
-        run_engine(e, program, 4)
-
-        # TODO look into logs - something is not right
+        run_engine(e, program, 3)
 
         system_state = e.tags[SystemTagName.SYSTEM_STATE]
         self.assertEqual(system_state.get_value(), SystemStateEnum.Restarting)
@@ -1395,13 +1349,13 @@ Restart
 
     def test_engine_error_causes_Paused_state(self):
         e = self.engine
-        with self.assertRaises(ValueError):
+        with self.assertRaises(EngineError):
             run_engine(e, "foo bar", 3)
         self.assertTrue(e.has_error_state())
 
     def test_interpreter_error_causes_Paused_state(self):
         e = self.engine
-        with self.assertRaises(ValueError):
+        with self.assertRaises(EngineError):
             run_engine(e, """WATCH x > 2""", 3)
         self.assertTrue(e._runstate_paused)
 
