@@ -28,6 +28,13 @@ class TestRunlog(unittest.TestCase):
         item_names = [item.name for item in runlog.items]
         self.fail(f"Runlog has no item named '{name}'. It has these names:  {','.join(item_names)}")
 
+    def assert_Runlog_HasNoItem(self, name: str):
+        runlog = self.engine.runtimeinfo.get_runlog()
+        item_names = [item.name for item in runlog.items]
+        for item in runlog.items:
+            if item.name == name:
+                self.fail(f"Runlog has item named '{name}' which was not expected. It has these names:  {','.join(item_names)}")
+
     def assert_Runlog_HasItem_Started(self, name: str):
         runlog = self.engine.runtimeinfo.get_runlog()
         for item in runlog.items:
@@ -284,6 +291,8 @@ Watch: Block Time > .3s
 
         self.assertEqual(item.forced, True)
         self.assertEqual(item.forcible, False)
+
+        continue_engine(e, 1)
         self.assertEqual(['Foo'], e.interpreter.get_marks())
 
     def test_iteration_modification(self):
@@ -374,6 +383,8 @@ Alarm: Block Time > .3s
 
         self.assertEqual(item.forced, True)
         self.assertEqual(item.forcible, False)
+
+        continue_engine(e, 1)
         self.assertEqual(['Foo'], e.interpreter.get_marks())
 
     def test_runlog_watch_in_alarm_body_runs_in_each_alarm_instance(self):
@@ -396,11 +407,11 @@ Alarm: Block Time > 0s
         continue_engine(e, 6)
         self.assertEqual(['A'], e.interpreter.get_marks())
 
-        continue_engine(e, 10)  # not sure how long to wait - but surely this is enough
+        continue_engine(e, 6)  # not sure how long to wait - but surely this is enough
         self.assert_Runlog_HasItem_Completed(alarm_item_name, 2)  # verify we waited long enough
         self.assertEqual(['A', 'A'], e.interpreter.get_marks())
 
-    def test_runlog_force_Mark_w_threshold_is_not_forcible(self):
+    def test_runlog_item_awaiting_threshold_is_not_rendered(self):
         e = self.engine
         program = """
 2 Mark: A
@@ -408,13 +419,7 @@ Alarm: Block Time > 0s
         run_engine(e, program, 4)
 
         mark_name = "Mark: A"
-        self.assert_Runlog_HasItem(mark_name)
-        runlog = e.runtimeinfo.get_runlog()
-        item = next(item for item in runlog.items if item.name == mark_name)
-        assert item is not None
-
-        self.assertEqual(item.forcible, False)
-        self.assertEqual(item.forced, False)
+        self.assert_Runlog_HasNoItem(mark_name)
 
     def test_runlog_force_Mark_without_threshold_is_not_forcible(self):
         e = self.engine
@@ -438,7 +443,7 @@ Mark: A
         cmd = "Wait: 0.5s"
         item_name = cmd
 
-        run_engine(e, cmd, 5)
+        run_engine(e, cmd, 3)
         print_runtime_records(e)
 
         self.assert_Runtime_HasRecord_Started(cmd)
@@ -447,8 +452,7 @@ Mark: A
         runlog = e.runtimeinfo.get_runlog()
         item = next((i for i in runlog.items if i.name == item_name), None)
         assert item is not None and item.progress is not None
-        # after 5-2 = 3 ticks we're 0.6 percent done
-        self.assertAlmostEqual(item.progress, 0.6, delta=0.05)
+        self.assertAlmostEqual(item.progress, 0.2, delta=0.1)
         self.assertEqual(item.forcible, True)
         self.assertEqual(item.cancellable, False)
 
@@ -456,8 +460,7 @@ Mark: A
         runlog = e.runtimeinfo.get_runlog()
         item = next((i for i in runlog.items if i.name == item_name), None)
         assert item is not None and item.progress is not None
-        # after 1 more tick we're 0.8 percent done
-        self.assertAlmostEqual(item.progress, 0.8, delta=0.05)
+        self.assertAlmostEqual(item.progress, 0.5, delta=0.1)
 
         continue_engine(e, 2)
         self.assert_Runlog_HasItem_Completed(cmd)
